from datetime import datetime, timedelta
from dateutil import tz

import s3_dumps.utils as utils

import os
import re
import logging

logger = logging.getLogger('s3_dumps')


class Archive(object):
    """
    Archives all backups on S3 using the following schedule:

    - Keep all backups for 7 days
    - Keep midnight backups for every other day for 30 days
    - Keep 1st day of the month forever
    """

    def __init__(self, conn, service_name, bucket, file_key, db_name=None):
        self.conn = conn
        self.service_name = service_name
        self.bucket = bucket
        self.file_key_suffix = utils.get_file_key(file_key=file_key, db_name=db_name)

    def archive(self):
        buck = self.conn.Bucket(self.bucket)
        for obj in buck.objects.filter(Prefix=self.file_key_suffix):
            if not obj.key.endswith("/"):
                name_parts = obj.key.split('/')
                month = name_parts[-2]
                year = name_parts[-3]
                new_key_name = obj.key

                if not re.match(r'[\d]{4}', year) and not re.match(r'[\d]{2}', month):
                    path, filename = os.path.split(obj.key)
                    new_key_name = '{path}/{year}/{month}/{day}/{filename}'.format(
                        path=path,
                        year=obj.last_modified.year,
                        month=obj.last_modified.month,
                        day=obj.last_modified.day,
                        filename=filename
                    )

                if self.remove_key(obj):
                    obj.delete()
                elif obj.key != new_key_name:
                    logger.info('Archiving object to {new_key}'.format(
                        key=obj.key,
                        new_key=new_key_name,
                    ))
                    copy_source = {'Bucket': obj.bucket_name, 'Key': obj.key}
                    new_obj = self.conn.Object(self.bucket, new_key_name)
                    new_obj.copy_from(CopySource=copy_source)
                    obj.delete()

    def remove_key(self, obj):
        delta = datetime.now().astimezone(tz=tz.tzutc()) - obj.last_modified
        week_delta = timedelta(days=7)
        month_delta = timedelta(days=30)

        if delta <= week_delta:
            logger.info('%s - Keeping object \"%s\" because it\'s less than a week old.' % (delta, obj.key))
            return False
        elif week_delta < delta < month_delta:
            if obj.last_modified.hour != 0:
                logger.info('%s - Removing object \"%s\" because it\'s not a midnight backup and it\'s older than one week but less than a month' % (delta, obj.key))
                return True
            elif obj.last_modified.day % 2 != 0:
                logger.info('%s - Removing object \"%s\" because it\'s older than one week but less than a month and not an even day.' % (delta, obj.key))
                return True
            else:
                logger.info('%s - Keeping object \"%s\" because it\'s older than one week but less than a month and it\'s an even day.' % (delta, obj.key))
                return False
        elif delta > month_delta:
            if obj.month_delta.day == 1:
                logger.info('%s - Keeping object \"%s\" because it\'s older than a month and also the first day of the month.' % (delta, obj.key))
                return False
            else:
                logger.info('%s - Removing object \"%s\" because it\'s older than a month and not the first day of the month.' % (delta, obj.key))
                return True



import boto3
import logging

logger = logging.getLogger('s3_dumps')


class s3Connect:
    """
    Connect to s3 using credentials
    Arguments:
        access_key_id -- Access Key ID
        secret_access_key -- Secret Acces Key
        region -- bucket region (default is set to '')
        service_name -- default is set to amazon (supported services - digitalocean and amazon)
    """

    def __init__(self, access_key_id, secret_access_key, region='', service_name='amazon'):
        self.access_key_id = access_key_id
        self.secret_access_key = secret_access_key
        self.region = region
        self.service_name = service_name

        if service_name == 'amazon':
            aws_base_url = 'https://s3.amazonaws.com'
            self.conn = boto3.resource('s3',
                                       region_name=region,
                                       endpoint_url=aws_base_url,
                                       aws_access_key_id=access_key_id,
                                       aws_secret_access_key=secret_access_key)
        elif self.service_name == 'digitalocean':
            do_base_url = 'https://%s.%s' % (region, 'digitaloceanspaces.com')
            self.conn = boto3.resource('s3',
                                       region_name=region,
                                       endpoint_url=do_base_url,
                                       aws_access_key_id=access_key_id,
                                       aws_secret_access_key=secret_access_key)
        else:
            raise Exception('Unsupported service name: %s', self.service_name)

    def get_conn(self):
        return self.conn

    def upload_file_to_cloud(self, bucket, media_location, file_key):
        """
        Uploads file to Cloud (Amazon or DigitalOcean) S3 services
        Arguments:
            media_location -- File location in local storage
            file_key -- file key, the directory path where the file is stored
            service -- 'amazon' for Amazon or 'digitalocean'
            bucket -- bucket name
        """
        logger.info('Uploading file key {} to Cloud ...'.format(file_key))
        buck = self.conn.Bucket(bucket)
        try:
            buck.put_object(
                Key=file_key,
                Body=open(media_location, 'rb'),
                ACL='public-read')
            logger.info('Uploaded file key {}.'.format(file_key))
        except Exception as e:
            logger.info('Error: ', e)



#!/usr/bin/env python

from datetime import datetime
from s3_dumps.connect import s3Connect
from s3_dumps.archive import Archive

import os
import sys
import argparse

import s3_dumps.utils as utils

import logging
import tarfile

logger = logging.getLogger('s3_dumps')


def create_redis_dump(command, filename):
    """
    Creates dump file using pg_dump
    Returns:
        1: Path of dump file from temp storage
    """
    if DUMP_BASE_DIR and not os.path.exists(DUMP_BASE_DIR):
        os.mkdir(DUMP_BASE_DIR)

    logger.info('Preparing ' + filename + '.sql from the database dump ...')
    logger.info('Executing: %s', command)
    ret = os.system(command)
    if ret != 0:
        sys.exit(ret)

    redis_dump_dir = REDIS_DUMP_DIR if REDIS_DUMP_DIR.endswith('/') else REDIS_DUMP_DIR + '/'

    tar = tarfile.open(DUMP_BASE_DIR + filename + '.tar.gz', 'w|gz')
    tar.add(redis_dump_dir + 'dump.rdb', filename + ".rdb")
    tar.close()
    logger.info('Created tar file ' + filename + '.tar.gz')
    return '{}{}.tar.gz'.format(DUMP_BASE_DIR, filename)


def backup():
    """Creates the buckup and uploads"""
    now = datetime.now()
    filename = ARCHIVE_NAME + now.strftime('_%Y%m%d_%H%M%S')

    command = REDIS_SAVE_CMD

    file_location = create_redis_dump(command, filename)
    file_key_suffix = utils.get_file_key(file_key=FILE_KEY)
    file_key = r'%s/%s' % (file_key_suffix, filename + '.tar.gz')

    conn.upload_file_to_cloud(
        bucket=BUCKET_NAME,
        media_location=file_location,
        file_key=file_key
    )

    logger.info('Sucessfully uploaded the Dump.')


if __name__ == '__main__':

    parser = argparse.ArgumentParser(description='''Creates redis DB dump and stores
                                     it to S3 using `redis-cli save`.''')
    parser = utils.init_arguments(parser)
    parser.add_argument('--ARCHIVE_NAME',
                        default='all_redis_db',
                        help='The base name for the archive')

    args = parser.parse_args()

    SERVICE_NAME = args.SERVICE_NAME
    ACCESS_KEY = args.ACCESS_KEY
    SECRET = args.SECRET
    REGION = args.REGION
    BUCKET_NAME = args.BUCKET_NAME
    FILE_KEY = args.FILE_KEY

    DUMP_BASE_DIR = args.DUMP_BASE_DIR

    REDIS_DUMP_DIR = args.REDIS_DUMP_DIR
    REDIS_SAVE_CMD = args.REDIS_SAVE_CMD
    ARCHIVE_NAME = args.ARCHIVE_NAME

    conn = s3Connect(access_key_id=ACCESS_KEY, secret_access_key=SECRET, region=REGION, service_name=SERVICE_NAME)

    if args.verbose:
        utils.init_logger(logger)

    if args.archive:
        archive = Archive(conn=conn.get_conn(), service_name=SERVICE_NAME,
                          bucket=BUCKET_NAME, file_key=FILE_KEY)
        archive.archive()

    if args.backup:
        backup()



#!/usr/bin/env python

from datetime import datetime
from s3_dumps.connect import s3Connect
from s3_dumps.archive import Archive

import os
import sys
import argparse

import s3_dumps.utils as utils

import logging
import tarfile

logger = logging.getLogger('s3_dumps')


def create_db_dump(command, filename):
    """
    Creates dump file using pg_dump
    Returns:
        1: Path of dump file from temp storage
    """
    if DUMP_BASE_DIR and not os.path.exists(DUMP_BASE_DIR):
        os.mkdir(DUMP_BASE_DIR)

    logger.info("Preparing " + filename + ".sql from the database dump ...")
    logger.info('Preparing ' + filename + '.sql from the database dump ...')
    logger.info('Executing: %s', command)
    ret = os.system(command)
    if ret != 0:
        sys.exit(ret)
    tar = tarfile.open(DUMP_BASE_DIR + filename + '.tar.gz', 'w|gz')
    tar.add(DUMP_BASE_DIR + filename + '.sql', filename + '.sql')
    tar.close()
    logger.info('Created tar file ' + filename + '.tar.gz')
    return '{}{}.tar.gz'.format(DUMP_BASE_DIR, filename)


def backup():
    """Creates the buckup and uploads"""
    now = datetime.now()
    archive_name = DB_NAME + '_db' if DB_NAME else ARCHIVE_NAME
    filename = archive_name + now.strftime('_%Y%m%d_%H%M%S')

    command = '{0} -h {1} -u {2} -p {3}'.format(MYSQL_DUMP_CMD, MYSQL_HOST,
                                                MYSQL_USER, MYSQL_PASSWORD)
    if DB_NAME:
        command += ' --databases {4} > {5}'.format(DB_NAME, DUMP_BASE_DIR + filename + '.sql')
    else:
        command += ' --all-databases > {5}'.format(DUMP_BASE_DIR + filename + '.sql')

    file_location = create_db_dump(command, filename)
    file_key_suffix = utils.get_file_key(file_key=FILE_KEY, db_name=DB_NAME)
    file_key = r'%s/%s' % (file_key_suffix, filename + '.tar.gz')

    conn.upload_file_to_cloud(
        bucket=BUCKET_NAME,
        media_location=file_location,
        file_key=file_key
    )

    if DELETE_DUMP:
        os.remove(file_location)
        logger.info('''Removed the dump from local directory ({}).'''.format(
            file_location))

    logger.info('Sucessfully uploaded the Dump.')


if __name__ == '__main__':

    parser = argparse.ArgumentParser(description='''Creates postgres DB
                                     dump and stores it to S3 using
                                     `pg_dump`.''')
    parser = utils.init_arguments(parser)

    parser.add_argument('--ARCHIVE_NAME',
                        default='all_mysql_db',
                        help='The base name for the archive')
    parser.add_argument('--MYSQL_DUMP_CMD',
                        default='mysqldump',
                        help="mysqldump command (default: mysqldump)")
    parser.add_argument('--MYSQL_HOST',
                        default='localhost',
                        help="Mysql host (default: localhost)")
    parser.add_argument('--MYSQL_USER',
                        default='root', help="Mysql user (default: root)")
    parser.add_argument('--MYSQL_PASSWORD',
                        default='', help="Mysql password (default: '')")

    args = parser.parse_args()

    SERVICE_NAME = args.SERVICE_NAME
    ACCESS_KEY = args.ACCESS_KEY
    SECRET = args.SECRET
    REGION = args.REGION
    BUCKET_NAME = args.BUCKET_NAME
    FILE_KEY = args.FILE_KEY

    DB_NAME = args.DB_NAME
    DELETE_DUMP = args.DELETE_DUMP
    DUMP_BASE_DIR = args.DUMP_BASE_DIR
    ARCHIVE_NAME = args.ARCHIVE_NAME

    MYSQL_DUMP_CMD = args.MYSQL_DUMP_CMD
    MYSQL_HOST = args.MYSQL_HOST
    MYSQL_USER = args.MYSQL_USER
    MYSQL_PASSWORD = args.MYSQL_USER

    conn = s3Connect(access_key_id=ACCESS_KEY, secret_access_key=SECRET, region=REGION, service_name=SERVICE_NAME)

    if args.verbose:
        utils.init_logger(logger)

    if args.archive:
        archive = Archive(conn=conn.get_conn(), service_name=SERVICE_NAME,
                          bucket=BUCKET_NAME, file_key=FILE_KEY, db_name=DB_NAME)
        archive.archive()

    if args.backup:
        backup()
